﻿using System;
using System.IO;
using System.Net;
using System.Web;
using System.Web.Script.Serialization;
using System.Web.UI.WebControls;

namespace ExorLiveClientScript
{
	public partial class _default : System.Web.UI.Page
	{
		// The client has been autorized by ExorLive API support for certain access rights.
		// The client is identified by the client id and secret.
		private const string clientID = "";			//TODO: Enter the client ID received from ExorLive API support.
													//TODO: The client ID shall also be entered in initExorLive.js, line 44.
		private const string clientSecret = "";		//TODO: Enter the client secret received from ExorLive API support.
		private const string applicationURL = "http://yourdomain.com/default.aspx";	//TODO: Enter the URL of your client application. Must match URL provided to ExorLive API support when client ID was issued.
		private const string scope = "read_profile read_workout create_session";  // TODO: Enter scope string (what your client application is allowed to do) as received from ExorLive API support.

		// Leave these 3 exactly as shown here.
		private const string loginURL = "https://auth.exorlive.com/Providers/OAuth/Authorize.aspx";
		private const string authPostURL = "https://auth.exorlive.com/Providers/OAuth/Token.ashx";
		private const string exorliveLibrary = "https://exorlive.com/api/bootstrapper.ashx";


		// Create a user object. This simulates a user of the client which is hooked up to a user in ExorLive.
		// Check out the implementation of this class. It simulates load/save of user data that allows for automatic re-login of the user in ExorLive when needed.
		LoggedInUser user = new LoggedInUser();

		protected void Page_Load(object sender, EventArgs e)
		{
			// Authorization of exorlive user login is using OAuth 2.0 standard.
			// It is a two-step process. First get a authorization code. Then use that code to get a access_token.
			// The access_token has a certain time it is valid before it expires. Then it may be renewed.

			// Check if we have an autorization code attached on the URL. If so, exchange it for an access_token.
			var code = Request.QueryString["code"];
			if (code != null && user.userToken.AccessToken == null)
			{
				string postData = String.Format("grant_type=authorization_code&client_id={0}&client_secret={1}&code={2}&redirect_uri={3}",
					clientID,
					clientSecret,
					code,
					HttpUtility.UrlEncode(applicationURL)
				);
				
				string response = doHttpWebRequest(authPostURL, postData, "POST");
				var serializer = new JavaScriptSerializer();
				var resp = serializer.Deserialize<dynamic>(response);

				user.userToken.AccessToken = resp["access_token"];
				user.userToken.RefreshToken = resp["refresh_token"];
				user.userToken.Type = resp["token_type"];
				user.userToken.Scope = resp["scope"];
				user.userToken.Expiration = DateTime.UtcNow.AddSeconds(Convert.ToInt32(resp["expires_in"])).ToString("yyyy-MM-dd HH:mm:ss");

				// Save this access information, so we can reuse it later and automatically log in.
				user.saveUserToken();
			}

			// Check if the customers access_token have expired.
			// If so, refresh it.
			if (user.userToken.Expiration != null && DateTime.Compare(DateTime.Parse(user.userToken.Expiration), DateTime.UtcNow) <= 0)
			{
				string postData = String.Format("grant_type=refresh_token&client_id={0}&client_secret={1}&refresh_token={2}&redirect_uri={3}",
					clientID,
					clientSecret,
					user.userToken.RefreshToken,
					HttpUtility.UrlEncode(applicationURL)
				);
				string response = doHttpWebRequest(authPostURL, postData, "POST");
				var serializer = new JavaScriptSerializer();
				dynamic resp = null;
				try
				{
					resp = serializer.Deserialize<dynamic>(response);
				}
				// This will happen when there is invalid token data.
				catch (ArgumentException)
				{
					user.clearUserToken();
				}

				if (resp != null)
				{
					user.userToken.AccessToken = resp["access_token"];
					user.userToken.RefreshToken = resp["refresh_token"];
					user.userToken.Type = resp["token_type"];
					user.userToken.Scope = resp["scope"];
					user.userToken.Expiration = DateTime.UtcNow.AddSeconds(Convert.ToInt32(resp["expires_in"])).ToString("yyyy-MM-dd HH:mm:ss");

					user.saveUserToken();
				}
			}
			// If I got a token and it is not expired, then I am authorized to continue. We then load the js library of ExorLive, if not loaded already.
			if (user.userToken.AccessToken != null)
			{
				ClientScript.RegisterClientScriptInclude("ExorLive", exorliveLibrary);
				ClientScript.RegisterClientScriptInclude("ExorLiveInit", "initExorLive.js");

				string jsAccessToken = String.Format("AccessToken:\"{0}\", Expiration: \"{1}\", Scope: \"{2}\"", user.userToken.AccessToken, user.userToken.Expiration, user.userToken.Scope);

				// We also put the access_token in the javascript so it can be used there.
				ClientScript.RegisterClientScriptBlock(this.GetType(), "access_token", "var $access_token = {" + jsAccessToken + "};", true);
			}

			// We simulate a simple menu to allow the user to disconnect/logout of ExorLive.
			// If the user is not connected to ExorLive, dont show the "disconnect" menu,
			// and do show the 'btnLoginExorLive' button.
			if (user.userToken.AccessToken != null)
			{
				menuClientApp.Visible = true;
				btnLoginExorLive.Visible = false;
			}
			else
			{
				menuClientApp.Visible = false;
				btnLoginExorLive.Visible = true;
			}

		}

		/// <summary>
		/// Does a HTTP web request.
		/// </summary>
		/// <param name="url">URL</param>
		/// <param name="varg">Arguments</param>
		/// <param name="method">POST or GET</param>
		/// <returns>Returns response from the request.</returns>
		private static string doHttpWebRequest(string url, string varg, string method)
		{
			string result = "";
			if (method == "GET")
			{
				url += "?" + varg;
			}
			HttpWebRequest req = (HttpWebRequest)WebRequest.Create(url);
			req.Method = method;

			if (req.Method == "POST")
			{
				req.ContentType = "application/x-www-form-urlencoded";
				// Webrequest body
				using (StreamWriter stream = new StreamWriter(req.GetRequestStream()))
				{
					stream.Write(varg);
				}
			}
			try
			{
				using (StreamReader stream = new StreamReader(req.GetResponse().GetResponseStream()))
				{
					result = stream.ReadToEnd();
				}
			}
			catch (WebException ex)
			{
				result = ex.Status.ToString();
				if (ex.Response.ContentLength != 0)
				{
					using (var stream = ex.Response.GetResponseStream())
					{
						using (var reader = new StreamReader(stream))
						{
							result += reader.ReadToEnd();
						}
					}
				}
			}
			return result;
		}

		/// <summary>
		/// Handles the MenuItemClick event of the simple menu.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="System.Web.UI.WebControls.MenuEventArgs"/> instance containing the event data.</param>
		protected void Menu1_MenuItemClick(object sender, MenuEventArgs e)
		{
			if (e.Item.Value == "ELDisconnect")
			{
				user.clearUserToken();
				Response.Redirect(applicationURL);
			}
		}

		/// <summary>
		/// Handles the Click event of the btnLoginExorLive control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		protected void btnLoginExorLive_Click(object sender, EventArgs e)
		{
			// This will redirect the user to the ExorLive login page. When we are logged in (or immediately, if we were already logged in) we are redirected back to here (the applicatioURL).
			// When we are redirected, an authorization code is attached hwich is handled in the top of  Page_Load above.
			string varg = String.Format("?response_type=code&client_id={0}&client_secret={1}&scope={2}&redirect_uri={3}", clientID, clientSecret, scope, HttpUtility.UrlEncode(applicationURL));
			Response.Redirect(loginURL + varg);
		}
	}

	/// <summary>
	/// Holds token data
	/// </summary>
	public class Token
	{
		public string AccessToken { get; set; }
		public string RefreshToken { get; set; }
		public string Expiration { get; set; }

		public string Scope { get; set; }

		public string Type { get; set; }
	}

	/// <summary>
	/// Simulates userdetails handling for a "logged in" user
	/// Saves all userdata to a file called dummy.txt
	/// in the temp folder.
	/// </summary>
	public class LoggedInUser
	{
		public Token userToken { get; set; }
		/// <summary>
		/// Initializes a new instance of the <see cref="LoggedInUser"/> class.
		/// </summary>
		public LoggedInUser()
		{
			userToken = new Token();
			loadUserToken();
		}
		/// <summary>
		/// Saves the user token.
		/// </summary>
		public void saveUserToken()
		{
			var serializer = new JavaScriptSerializer();
			string output = serializer.Serialize(this.userToken);
			TextWriter stream = new StreamWriter(Path.GetTempPath() + "dummy.txt");
			stream.Write(output);
			stream.Close();
		}
		/// <summary>
		/// Clears the user token.
		/// </summary>
		public void clearUserToken()
		{
			string path = Path.GetTempPath() + "dummy.txt";
			File.Delete(path);
			userToken = new Token();
		}
		/// <summary>
		/// Loads the user token.
		/// </summary>
		public void loadUserToken()
		{
			string path = Path.GetTempPath() + "dummy.txt";
			if (File.Exists(path))
			{
				using (TextReader stream = new StreamReader(path))
				{
					string input = stream.ReadToEnd();
					var serializer = new JavaScriptSerializer();
					var dInput = serializer.Deserialize<dynamic>(input);
					userToken = new Token();
					userToken.AccessToken = dInput["AccessToken"];
					userToken.RefreshToken = dInput["RefreshToken"];
					userToken.Type = dInput["Type"];
					userToken.Scope = dInput["Scope"];
					userToken.Expiration = dInput["Expiration"];
				}
			}
			else
			{
				// If there is no file with user details,
				// just create a blank token.
				userToken = new Token();
			}
		}
	}
}