﻿//
// The ExorLive bootstrapper (api/bootstrapper.ashx) is the ExorLive framework.
// It must be loaded in this webpage. In this example it is loaded by default.aspx.
// The framework uses easyXDM to communicate between this client (on your domain) and the exorlive application in another tab.
//
// The framework will call init_ExorLive() below as soon as it is loaded.
// In this method you may customize the behavior of the link between the client and the exorlive application.
//
function init_ExorLive() {

    // This is an event that may come from exorlive. You may choose to handle it.
    ExorLive.Events.on("access.code", function (code) {
        if (typeof console != "undefined" && console.log) {
            console.log("** event(access.code)");
        }
	});

    // This is an event that may come from exorlive. You may choose to handle it.
    ExorLive.Events.on("access.expired", function () {
        // This may happen if you make a request to exorlive when the token has expired.
        if (typeof console != "undefined" && console.log) {
            console.log("** event(access.expired)");
        }

        // You may choose to implement an ajax call to the backend to refresh the token.
        // Like this:
        //    myajaxframework.RefreshAccessToken(function(accessToken) {
        //        if(accessToken)
        //        {
        //           ExorLive.setAccessToken(accessToken);
        //        }
        //    }

        // If your backend code keeps track of the expiretime and does the refresh in time, 
        // (like in the backend code of this example) then handling this event may not be needed.
	});

    // This is an event that may come from exorlive. You may choose to handle it.
    ExorLive.Events.on("access.missing", function (err) {
        // This event happens when the framework is initialized before we have a valid access_token.
        if (typeof console != "undefined" && console.log) {
            console.log("** event(access.missing)");
        }
	});

    // This is an event that may come from exorlive. You may choose to handle it.
    ExorLive.Events.on("auth.success", function (profile) {
        // We have just authorized and got a valid access_token.
        if (typeof console != "undefined" && console.log) {
            console.log("** event(auth.success)");
        }
        // If the user is just returning from a login, 
		// redirect him to ExorLive with the customer data.
        // But wait until ExorLive is ready before running this.
        // In this example we use a cookie to remember the contact we want to open in ExorLive.
		var customerIdToInstantlyOpen = JSON.parse(readCookie('redirectToCustomer'));
		if (customerIdToInstantlyOpen != null) {
			eraseCookie('redirectToCustomer');
			document.getElementById('dlg2').style.visibility = '';
			document.getElementById('btnOpenCustomer2').onclick = function () { btnOpenMyCustomer(customerIdToInstantlyOpen); return false; };
		}
	});

	// This is where you configure the API
	ExorLive.init({

	    // The culture you want used
	    culture: "nb-NO", // Leave blank for default for the user.

		// Your ExorLive Applications identifier
		clientId: "<fill in>", // Fill in your clientId that you got from ExorLive API support.


	    // If the user has already autorized you, then you can supply the token here.
	    // The accessToken was generated by the serverside and added to the html document
		// as a javascript global called $access_token.
		accessToken: typeof $access_token == "undefined" ? null : $access_token,
		// Whether the API should look for, and instantiate ExorLive Widgets
		parse: true

	    /* The content of the token looks like this:
		accessToken: {
			// The token string
			AccessToken: "...................",
			// ISO8601 using UTC
			Expiration: "2011-09-21 12:33:10",
			// The scopes authorized through this token. This will help the API give proper feedback.
			Scope: "read_profile read_workouts ...."
		},*/

	});
}


